#include "mainwindow.h"
#include "mythread.h"
#include <QTextStream>
#include <QMessageBox>

static QString lesA = "ÀÂÅÃÄÁ";
static QString mesA = "AAAAAA";
static QString lesE = "ÉÈÊË";
static QString mesE = "EEEE";
static QString lesI = "ÎÏÍ";
static QString mesI = "III";
static QString lesO = "ÔÓÖ";
static QString mesO = "OOO";
static QString lesU = "ÙÛÜ";
static QString mesU = "UUU";
static QString lesY = "Ÿ";
static QString mesY = "Y";
static QString lesC = "Ç";
static QString mesC = "C";
static QString lesN = "NÑ";
static QString mesN = "NN";
static QString lesS = "S";
static QString mesS = "S";

static QString strOri = lesA + lesE + lesI + lesO + lesU + lesY + lesC + lesN + lesS;
static QString strRep = mesA + mesE + mesI + mesO + mesU + mesY + mesC + mesN + mesS;

// Mise en MAJUSCULE et SUPPRESSION des "-", ".", "'"
static  QString sepa = " -.'";
QString compacte(QString line)
{
    QChar mot[80];
    int lm = 0;
    for(int i = 0; i < line.length(); i++)
    {
        QChar c = line[i];
        if (!sepa.contains(c)) mot[lm++] = c.toUpper();
    }
    return QString(mot, lm);
}

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
    select = new QPalette(QWidget::palette());
    nonsel = new QPalette(QWidget::palette());
    select->setBrush(QPalette::Button, select->highlight());
    select->setBrush(QPalette::ButtonText, select->highlightedText());
    setWindowTitle("Scrabble");
    // Le layout principal est vertical
    QWidget *centralWidget = new QWidget(this);
    setCentralWidget(centralWidget);
    QVBoxLayout *layoutPpalV = new QVBoxLayout(centralWidget);
    // Ligne 0 : Bouton(Edition dictionnaire) + CheckBox(Concordance stricte)
    QPushButton *button_EditDico = new QPushButton(centralWidget);
    button_EditDico->setText("Editer mon dictionnaire");
    layoutPpalV->addWidget(button_EditDico);
    connect(button_EditDico, SIGNAL(clicked(bool)), this, SLOT(on_pushButton_EditDico_clicked()));
    // 1ere Ligne : Bouton(Scrable) + Bouton(Mots Croisés) + CheckBox(Concordance stricte)
    QHBoxLayout *layoutH = new QHBoxLayout();
    button_Scrable = new QPushButton(centralWidget);
    button_Scrable->setText("Scrable");
    button_Scrable->setPalette(*select);
    button_Xword = new QPushButton(centralWidget);
    button_Xword->setText("Mots Xsés");
    checkBox_Concorde = new QCheckBox(centralWidget);
    checkBox_Concorde->setText("Accent strict");
    layoutH->addWidget(button_Scrable);
    layoutH->addWidget(button_Xword);
    layoutH->addWidget(checkBox_Concorde);
    layoutPpalV->addLayout(layoutH);
    connect(button_Scrable, SIGNAL(clicked(bool)), this, SLOT(on_button_Scrable()));
    connect(button_Xword, SIGNAL(clicked(bool)), this, SLOT(on_button_Xword()));
    connect(checkBox_Concorde, SIGNAL(stateChanged(int)), this, SLOT(on_checkBox_stateChanged(int)));
    // 2eme Ligne : Label(Lettres autorisees) + TextEdit(azertyuiop)
    layoutH = new QHBoxLayout();
    labelProto = new QLabel(centralWidget);
    labelProto->setText("Lettres autorisées");
    lettresEdit = new QLineEdit(centralWidget);
    lettresEdit->setText("azertyuiop");
    layoutH->addWidget(labelProto);
    layoutH->addWidget(lettresEdit);
    layoutPpalV->addLayout(layoutH);
    connect(lettresEdit, SIGNAL(editingFinished()), this, SLOT(on_lineEdit_editingFinished()));
    // 3eme ligne : Label(Nb de lettes) + spinBox(Nb de lettres)
    layoutH = new QHBoxLayout();

    labelMots = new QLabel(centralWidget);
    labelMots->setText("Mots de combien de lettres ?");

    spinBox = new QSpinBox(centralWidget);
    spinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    spinBox->setValue(5);

    layoutH->addWidget(labelMots);
    layoutH->addWidget(spinBox);
    layoutPpalV->addLayout(layoutH);
    connect(spinBox, SIGNAL(editingFinished()), this, SLOT(on_spinBox_editingFinished()));
    // 4eme ligne Bouton(Go) + Bouton(Cancel) + LineEdit(nb de mots trouvés)
    layoutH = new QHBoxLayout();
    QPushButton *pushButton_Go = new QPushButton(centralWidget);
    pushButton_Go->setText("Go");
    QPushButton *pushButton_Cancel = new QPushButton(centralWidget);
    pushButton_Cancel->setText("Cancel");
    lineEdit_Result = new QLineEdit(centralWidget);
    lineEdit_Result->setMouseTracking(false); // Ne capture pas la souris
    lineEdit_Result->setAcceptDrops(false); // N'accepte pas les dépots
    lineEdit_Result->setReadOnly(true); // En lecture seule
    lineEdit_Result->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
    lineEdit_Result->setText("0");
    layoutH->addWidget(pushButton_Go);
    layoutH->addWidget(pushButton_Cancel);
    layoutH->addWidget(lineEdit_Result);
    layoutPpalV->addLayout(layoutH);
    connect(pushButton_Go, SIGNAL(clicked(bool)), this, SLOT(on_pushButton_Go_clicked()));
    connect(pushButton_Cancel, SIGNAL(clicked(bool)), this, SLOT(on_pushButton_Cancel_clicked()));
    // 5eme ligne : Un progressBar
    progressBar = new QProgressBar(centralWidget);

    progressBar->setTextVisible(false);
    layoutPpalV->addWidget(progressBar);
    // Tout le reste en dessous : le TextEdit des mots solutions
    textEdit = new QTextEdit(centralWidget);
    textEdit->setMouseTracking(false); // Ne capture pas la souris
    textEdit->setAcceptDrops(false); // N'accepte pas les dépots
    textEdit->setReadOnly(true);
    layoutPpalV->addWidget(textEdit);
    ///////////////////////////////////////////////////////////////////
    QString line, majline;
    filein = new QFile(":/frdico/dicofr",this);
    if (!filein->open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QMessageBox::warning(this,"ERREUR OPEN","dicofr absent");
        return;
    }
    in = new QTextStream(filein);
    in->setCodec("UTF-8");
    while (true)
    {
        line = in->readLine();
        if(line.isNull()) break;
        mindico1.append(line);
        majline = compacte(line);
        majdico1.append(majline);
    }
    filein->close();
    delete in;
    delete filein;
    if (majdico1.size() != mindico1.size())
    {
        QMessageBox::warning(this,"ERREUR DICO1",tr("%1 mot(s) incohérent(s)").arg(mindico1.size()-majdico1.size()));
        exit(1);
    }

    filein = new QFile("mondico.txt",this);
    if (filein->open(QIODevice::ReadWrite | QIODevice::Text))
    {
        in = new QTextStream(filein);
        in->setCodec("UTF-8");
        while (true)
        {
            line = in->readLine();
            if(line.isNull()) break;
            mindico2.append(line);
            majline = compacte(line);
            majdico2.append(majline);
        }
        if (majdico2.size() != mindico2.size())
        {
            QMessageBox::warning(this,"ERREUR DICO2",tr("%1 mot(s) incohérent(s)").arg(mindico2.size()-majdico2.size()));
            exit(1);
        }
    }
    else
    {
        QMessageBox::warning(this,"ATTENTION","Erreur acces mondico.txt");
        delete filein;
        filein = NULL;
    }
    dicSize = mindico1.size() + mindico2.size();
    progressBar->setRange(0, dicSize);
    progressBar->setValue(0);

    modifs = false;
    synonyme = true;
    checkBox_Concorde->setChecked(!synonyme);

    if (filein)
    {
        myEditDialog = new EditDialog(&mindico2, this);
        myEditDialog->setWindowFlags(Qt::Window);
    }
    else myEditDialog = NULL;
    recherche = new MyThread(this, this);
    connect(recherche, SIGNAL(avance(int)), this, SLOT(aff_avance(int)));
    connect(recherche, SIGNAL(trouve(int)), this,SLOT(aff_trouve(int)));
    connect(recherche, SIGNAL(finished()), this, SLOT(aff_solution()));
}

MainWindow::~MainWindow()
{
    if (modifs)
    {
        int rep = QMessageBox::question(this,"CHMOTS","Voulez-vous sauvegarder vos modifications ?", QMessageBox::Yes, QMessageBox::No);

        if (rep == QMessageBox::Yes)
        {
            filein->resize(0);
            in->seek(0);
            for(int i = 0; i < mindico2.size(); i++) *in << mindico2.at(i) << "\n";
            in->flush();
            filein->close();
        }
    }
}

void MainWindow::aff_avance(int i)
{
    progressBar->setValue(i);
}

void MainWindow::aff_trouve(int i)
{
    lineEdit_Result->setText(QString::number(i));
}

void MainWindow::aff_solution()
{
    textEdit->setText(solution);
    progressBar->reset();
}

void MainWindow::on_button_Scrable()
{
    labelMots->setEnabled(true);
    labelProto->setText("Lettres autorisées");
    spinBox->setEnabled(true);
    spinBox->setMinimum(1);
    spinBox->setMaximum(24);
    spinBox->setValue(1);
    setWindowTitle("Scrable");
    textEdit->setText("Dans ""Lettres autorisées"" placer la liste des lettres autorisées. Cocher la case"
                      " ""Accent strict"" si les lettres accentuées doivent être respectées");
    lettresEdit->setText("azertyuiop");

    button_Scrable->setPalette(*select);
    button_Xword->setPalette(*nonsel);
}

void MainWindow::on_button_Xword()
{
    labelMots->setEnabled(false);
    labelProto->setText("Lettres connues");
    spinBox->setMinimum(0);
    spinBox->setMaximum(0);
    spinBox->setValue(0);
    spinBox->setEnabled(false);
    setWindowTitle("Mots croisés");
    textEdit->setText("Dans ""Lettres connues"" placer un chiffre, un point (.) ou une * à la place des lettres inconnues");
    lettresEdit->setText("");

    button_Scrable->setPalette(*nonsel);
    button_Xword->setPalette(*select);
}
void MainWindow::on_pushButton_Go_clicked()
{
    // Nb de lettres du mot recherché (si 0, cas mot croisé, c'est nbcharautor qui compte)
    solution.clear();
    progressBar->setValue(0);
    nbautor = spinBox->value();
    autorises = lettresEdit->text();

    if(recherche->isRunning()) return;
    cancel = false;
    lineEdit_Result->setText(QString::number(0));
    textEdit->setText("");
    recherche->start();
    progressBar->setValue(0);
}


void MainWindow::on_lineEdit_editingFinished()
{
    QString autorises = lettresEdit->text();
    QString good("");
    if (spinBox->value() == 0)
    {
        for (int i = 0; i < autorises.length(); i++)
        {
            QChar c = autorises.at(i);
            if (c.isLetter() || c == '*') good.append(c.toUpper());
            if (c.isDigit() || c == '.') good.append('*');
        }
    }
    else
    {
        for (int i = 0; i < autorises.length(); i++)
            if (autorises.at(i).isLetter()) good.append(autorises.at(i).toUpper());
    }
    lettresEdit->setText(good);
}

void MainWindow::on_spinBox_editingFinished()
{
    int nbletter = lettresEdit->text().length();
    int n = spinBox->value();
    if (n > nbletter) { n = nbletter; spinBox->setValue(n); }
    if (n < 0) { n = 0; spinBox->setValue(n); }
    if (n == 0) setWindowTitle("Mots croisés");
    else setWindowTitle("Scrabble");
}

void MainWindow::on_pushButton_EditDico_clicked()
{
    if (myEditDialog)
    {
        this->hide();
        myEditDialog->exec(); // exec -> MODAL
        this->show();
        int ok = myEditDialog->result();
        if (ok == QDialog::Accepted)
        {
            modifs = true;
            majdico2.clear();
            for (int i = 0; i < mindico2.size(); i++)
                majdico2.append(compacte(mindico2.at(i)));
            if (majdico2.size() != mindico2.size())
            {
                QMessageBox::warning(this,"ERREUR DICO2",tr("%1 mot(s) incohérent(s)").arg(mindico2.size()-majdico2.size()));
                exit(1);
            }
            dicSize = mindico1.size() + mindico2.size();
        }
    }
}

void MainWindow::on_checkBox_stateChanged(int arg1)
{
    synonyme = !checkBox_Concorde->isChecked();
}

void MainWindow::on_pushButton_Cancel_clicked()
{
    cancel = true;
}
